@@ -57,7 +57,7 @@ round(bst_preds_logodds,2) == round(bst_from_json_logodds,2)
 
 ## -----------------------------------------------------------------------------
 # now convert the dates to floats first
-bst_from_json_logodds <- ifelse(as.float(data$dates)<node$split_condition,
+bst_from_json_logodds <- ifelse(fl(data$dates)<node$split_condition,
                                 node$children[[1]]$leaf,
                                 node$children[[2]]$leaf)
 
@@ -65,7 +65,7 @@ bst_from_json_logodds <- ifelse(as.float(data$dates)<node$split_condition,
 round(bst_preds_logodds,2) == round(bst_from_json_logodds,2)
 
 ## -----------------------------------------------------------------------------
-as.float(20180131)
+fl(20180131)
 
 ## -----------------------------------------------------------------------------
 # test that values are equal
@@ -73,9 +73,9 @@ bst_preds_logodds == bst_from_json_logodds
 
 ## -----------------------------------------------------------------------------
 # now convert the dates to floats first
-bst_from_json_logodds <- ifelse(as.float(data$dates)<as.float(node$split_condition),
-                                as.numeric(as.float(node$children[[1]]$leaf)),
-                                as.numeric(as.float(node$children[[2]]$leaf)))
+bst_from_json_logodds <- ifelse(fl(data$dates)<fl(node$split_condition),
+                                as.numeric(fl(node$children[[1]]$leaf)),
+                                as.numeric(fl(node$children[[2]]$leaf)))
 
 # test that values are equal
 bst_preds_logodds == bst_from_json_logodds
@@ -84,9 +84,9 @@ bst_preds_logodds == bst_from_json_logodds
 bst_preds <- predict(bst,as.matrix(data$dates))
 
 # calculate the predictions casting doubles to floats
-bst_from_json_preds <- ifelse(as.float(data$dates)<as.float(node$split_condition),
-                              as.numeric(1/(1+exp(-1*as.float(node$children[[1]]$leaf)))),
-                              as.numeric(1/(1+exp(-1*as.float(node$children[[2]]$leaf))))
+bst_from_json_preds <- ifelse(fl(data$dates)<fl(node$split_condition),
+                              as.numeric(1/(1+exp(-1*fl(node$children[[1]]$leaf)))),
+                              as.numeric(1/(1+exp(-1*fl(node$children[[2]]$leaf))))
 )
 
 # test that values are equal
@@ -94,9 +94,9 @@ bst_preds == bst_from_json_preds
 
 ## -----------------------------------------------------------------------------
 # calculate the predictions casting doubles to floats
-bst_from_json_preds <- ifelse(as.float(data$dates)<as.float(node$split_condition),
-                              as.numeric(as.float(1)/(as.float(1)+exp(as.float(-1)*as.float(node$children[[1]]$leaf)))),
-                              as.numeric(as.float(1)/(as.float(1)+exp(as.float(-1)*as.float(node$children[[2]]$leaf))))
+bst_from_json_preds <- ifelse(fl(data$dates)<fl(node$split_condition),
+                              as.numeric(fl(1)/(fl(1)+exp(fl(-1)*fl(node$children[[1]]$leaf)))),
+                              as.numeric(fl(1)/(fl(1)+exp(fl(-1)*fl(node$children[[2]]$leaf))))
 )
 
 # test that values are equal
